==> tdf?file=1&path=frealign_v8.11_130826.tar.gz: frealign_v8.11/src/ctfapply.f <==
C**************************************************************************
      SUBROUTINE CTFAPPLY(NSAM,SPEC,SPEQ,SHX,SHY,CS,WL,WGH1,
     +		 WGH2,DFMID1,DFMID2,ANGAST,THETATR,CTFF,CTFS,OUTD,
     +		 OUTC,OUTQ,AMAGP,RIH,HALFW,RI2,RI3,RI4,
     +		 DATD,DATC,DATQ,B3DV,PHI,THETA,PSI,W,XSTD,IBUF,
     +           TX,TY,ASYM)
C**************************************************************************
C  Applies calculated CTF to a particle transform and also masks the 
C  particle in real space with a cosine-bell-edged circle of radius RI
C  Calls rlft3 and MASKENV.
C  Uses function CTF.
C  Used in LMAIN.
C**************************************************************************
      IMPLICIT NONE

      INTEGER NSAM,JC,L,LL,M,MM,ID,I,II,NMID
      INTEGER ISUM,NSAMH,IBUF,IC
      REAL PHASE,OUTD(*),FI1,AMAGP,CRAD2,SRAD2,RIH,W(*)
      REAL RI2,RI3,HALFW,PSHFTR,DATD(*),PI,TX,TY
      REAL RI4,SHX,SHY,CS,WL,WGH1,WGH2,DFMID1,DFMID2
      REAL THETATR,THETATRP,ANGAST,PX
      REAL RIHP,SCAL,DF1OLD,DF2OLD,ANGOLD
      REAL B3DV(*),PHI,THETA,PSI,XSTD
      PARAMETER  (PI=3.1415926535897)
      COMPLEX CTF,CTFR,CTFL,CTFF(*),CTFS(*)
      COMPLEX PSHFT,SPEC(*),SPEQ(*),OUTC(*),OUTQ(*)
      COMPLEX DATC(*),DATQ(*)
      CHARACTER ASYM*3
      LOGICAL NEWCTF
      SAVE DF1OLD,DF2OLD,ANGOLD
      DATA DF1OLD,DF2OLD,ANGOLD/0.0,0.0,0.0/
C**************************************************************************
C      write(*,*)'Now inside CTFAPPLY',AMAGP
      THETATRP=THETATR/AMAGP
      RIHP=RIH/AMAGP
      SCAL=NSAM*NSAM/2
C	  THETATRP is the diffraction angle for the first pixel in 
C	  the transform of a particle sampled at the input pixel size
      JC=NSAM/2+1
      NSAMH=NSAM/2
      NMID=NSAM/2+1
      IC=NSAM*NSAM/2
C     CORRECT FOR CTF AND APPLY SHIFTS
      NEWCTF=.FALSE.
      IF ((DFMID1.NE.DF1OLD).OR.
     +    (DFMID2.NE.DF2OLD).OR.
     +    (ANGAST.NE.ANGOLD)) NEWCTF=.TRUE.
C
      IF (NEWCTF) THEN
        DF1OLD=DFMID1
        DF2OLD=DFMID2
        ANGOLD=ANGAST
C        print *,'NEW CTF'
C
      DO 111 L=1,JC
        LL=L-1
        PX=SHX*LL
        DO 111 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
          PHASE=PX+SHY*MM
          PSHFT=CMPLX(COS(PHASE),SIN(PHASE))
          CTFR=CTF(CS,WL,WGH1,WGH2,DFMID1,DFMID2,
     +             ANGAST,THETATRP,LL,MM,TX,TY)
          CTFL=CTF(CS,WL,WGH1,WGH2,DFMID1,DFMID2,
     +             ANGAST,THETATRP,-LL,-MM,TX,TY)
          IF (L.NE.JC) THEN
      	    ID=L+NSAMH*(M-1)
            OUTC(ID)=SPEC(ID)*(CTFR+CONJG(CTFL))*PSHFT
            DATC(ID)=SPEC(ID)*PSHFT
      	    CTFF(ID)=CTFR
      	    CTFF(ID+IC)=CTFL
          ELSE
            OUTQ(M)=SPEQ(M)*(CTFR+CONJG(CTFL))*PSHFT
            DATQ(M)=SPEQ(M)*PSHFT
      	    CTFS(M)=CTFR
      	    CTFS(M+NSAM)=CTFL
          ENDIF
111   CONTINUE
C
      ELSE
C
      DO 112 L=1,JC
        LL=L-1
        PX=SHX*LL
        DO 112 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
          PHASE=PX+SHY*MM
          PSHFT=CMPLX(COS(PHASE),SIN(PHASE))
          IF (L.NE.JC) THEN
      	    ID=L+NSAMH*(M-1)
      	    CTFR=CTFF(ID)
      	    CTFL=CTFF(ID+IC)
            OUTC(ID)=SPEC(ID)*(CTFR+CONJG(CTFL))*PSHFT
            DATC(ID)=SPEC(ID)*PSHFT
          ELSE
      	    CTFR=CTFS(M)
      	    CTFL=CTFS(M+NSAM)
            OUTQ(M)=SPEQ(M)*(CTFR+CONJG(CTFL))*PSHFT
            DATQ(M)=SPEQ(M)*PSHFT
          ENDIF
112   CONTINUE
C
      ENDIF
C
      CALL RLFT3(OUTD,OUTQ,NSAM,NSAM,1,-1)
      DO 21 I=1,NSAM*NSAM
        OUTD(I)=OUTD(I)/SCAL
21    CONTINUE
C
C     MASK PARTICLE, COSINE EDGE
      IF (ASYM(1:1).EQ.'H') THEN
        CALL MASKCOS_C(NSAM,OUTD,RI2,RI3,RIH,
     +                 HALFW,AMAGP,PSI)
      ELSE
        CALL MASKCOS(NSAM,OUTD,RI2,RI3,RIH,
     +               HALFW,AMAGP)
      ENDIF

      IF (XSTD.LT.0.0) THEN
C	Apply tight mask to image used for refinement.
      	CALL MASKENV(NSAM,RIHP,OUTD,B3DV,0.0,0.0,
     +               PHI,THETA,PSI,W,IBUF)
      ENDIF

      CALL RLFT3(OUTD,OUTQ,NSAM,NSAM,1,1)
C      write(*,*)'after RLFT3s'

C     SHIFT PARTICLE TO ORIGIN
      DO 110 L=1,JC
        LL=L-1
        DO 110 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
      	  ISUM=(LL+MM)
      	  PSHFTR=1.0
      	  IF (MOD(ISUM,2).NE.0) PSHFTR=-1.0
          IF (L.NE.JC) THEN
      	    ID=L+NSAMH*(M-1)
            DATC(ID)=DATC(ID)*PSHFTR
            OUTC(ID)=OUTC(ID)*PSHFTR
          ELSE
            DATQ(M)=DATQ(M)*PSHFTR
            OUTQ(M)=OUTQ(M)*PSHFTR
          ENDIF
110   CONTINUE

      RETURN
      END

==> tdf?file=1&path=frealign_v9.05_131002.tar.gz: frealign_v9.05/src/ctfapply.f <==
C*
C* Copyright 2013 Howard Hughes Medical Institute.
C* All rights reserved.
C* Use is subject to Janelia Farm Research Campus Software Copyright 1.1
C* license terms ( http://license.janelia.org/license/jfrc_copyright_1_1.html )
C*
C**************************************************************************
      SUBROUTINE CTFAPPLY(NSAM,SPEC,SPEQ,SHX,SHY,CS,WL,WGH1,
     +		 WGH2,DFMID1,DFMID2,ANGAST,THETATR,CTFF,CTFS,OUTD,
     +		 OUTC,OUTQ,AMAGP,RIH,HALFW,RI2,RI3,RI4,
     +		 DATD,DATC,DATQ,B3DV,PHI,THETA,PSI,W,XSTD,IBUF,
     +           TX,TY)
C**************************************************************************
C  Applies calculated CTF to a particle transform and also masks the 
C  particle in real space with a cosine-bell-edged circle of radius RI
C  Calls rlft3 and MASKENV.
C  Uses function CTF.
C  Used in LMAIN.
C**************************************************************************
      IMPLICIT NONE

      INTEGER NSAM,JC,L,LL,M,MM,ID,I,II,NMID
      INTEGER ISUM,NSAMH,IBUF,IC
      REAL PHASE,OUTD(*),FI1,AMAGP,CRAD2,SRAD2,RIH,W(*)
      REAL RI2,RI3,HALFW,PSHFTR,DATD(*),PI,TX,TY
      REAL RI4,SHX,SHY,CS,WL,WGH1,WGH2,DFMID1,DFMID2
      REAL THETATR,THETATRP,ANGAST,PX
      REAL RIHP,SCAL,DF1OLD,DF2OLD,ANGOLD
      REAL B3DV(*),PHI,THETA,PSI,XSTD
      PARAMETER  (PI=3.1415926535897)
      COMPLEX CTF,CTFR,CTFL,CTFF(*),CTFS(*)
      COMPLEX PSHFT,SPEC(*),SPEQ(*),OUTC(*),OUTQ(*)
      COMPLEX DATC(*),DATQ(*)
      LOGICAL NEWCTF
      SAVE DF1OLD,DF2OLD,ANGOLD
      DATA DF1OLD,DF2OLD,ANGOLD/0.0,0.0,0.0/
C**************************************************************************
C      write(*,*)'Now inside CTFAPPLY',AMAGP
      THETATRP=THETATR/AMAGP
      RIHP=RIH/AMAGP
      SCAL=NSAM*NSAM/2
C	  THETATRP is the diffraction angle for the first pixel in 
C	  the transform of a particle sampled at the input pixel size
      JC=NSAM/2+1
      NSAMH=NSAM/2
      NMID=NSAM/2+1
      IC=NSAM*NSAM/2
C     CORRECT FOR CTF AND APPLY SHIFTS
      NEWCTF=.FALSE.
      IF ((DFMID1.NE.DF1OLD).OR.
     +    (DFMID2.NE.DF2OLD).OR.
     +    (ANGAST.NE.ANGOLD)) NEWCTF=.TRUE.
C
      IF (NEWCTF) THEN
        DF1OLD=DFMID1
        DF2OLD=DFMID2
        ANGOLD=ANGAST
C
      DO 111 L=1,JC
        LL=L-1
        PX=SHX*LL
        DO 111 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
          PHASE=PX+SHY*MM
          PSHFT=CMPLX(COS(PHASE),SIN(PHASE))
          CTFR=CTF(CS,WL,WGH1,WGH2,DFMID1,DFMID2,
     +             ANGAST,THETATRP,LL,MM,TX,TY)
          CTFL=CTF(CS,WL,WGH1,WGH2,DFMID1,DFMID2,
     +             ANGAST,THETATRP,-LL,-MM,TX,TY)
          IF (L.NE.JC) THEN
      	    ID=L+NSAMH*(M-1)
            OUTC(ID)=SPEC(ID)*(CTFR+CONJG(CTFL))*PSHFT
            DATC(ID)=SPEC(ID)*PSHFT
      	    CTFF(ID)=CTFR
      	    CTFF(ID+IC)=CTFL
          ELSE
C      	IF(M.LT.0.OR.M.GT.NSAM) WRITE(*,*)'M=',M
            OUTQ(M)=SPEQ(M)*(CTFR+CONJG(CTFL))*PSHFT
            DATQ(M)=SPEQ(M)*PSHFT
      	    CTFS(M)=CTFR
      	    CTFS(M+NSAM)=CTFL
          ENDIF
111   CONTINUE
C
      ELSE
C
      DO 112 L=1,JC    
        LL=L-1
        PX=SHX*LL
        DO 112 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
          PHASE=PX+SHY*MM
          PSHFT=CMPLX(COS(PHASE),SIN(PHASE))
          IF (L.NE.JC) THEN
            ID=L+NSAMH*(M-1)
            CTFR=CTFF(ID)
            CTFL=CTFF(ID+IC)
            OUTC(ID)=SPEC(ID)*(CTFR+CONJG(CTFL))*PSHFT
            DATC(ID)=SPEC(ID)*PSHFT
          ELSE
            CTFR=CTFS(M)
            CTFL=CTFS(M+NSAM)   
            OUTQ(M)=SPEQ(M)*(CTFR+CONJG(CTFL))*PSHFT
            DATQ(M)=SPEQ(M)*PSHFT
          ENDIF
112   CONTINUE
C
      ENDIF
C
      CALL RLFT3(OUTD,OUTQ,NSAM,NSAM,1,-1)
      DO 21 I=1,NSAM*NSAM
        OUTD(I)=OUTD(I)/SCAL
C        DATD(I)=OUTD(I)
21    CONTINUE
C
C     MASK PARTICLE, COSINE EDGE
      CALL MASKCOS(NSAM,OUTD,RI2,RI3,RIH,HALFW,AMAGP)
C     No masking for particles that are used for reconstruction
C     since this would cut out some CTF fringes and produce
C     ripples in transform
C      CALL MASKCOS(NSAM,DATD,RI4,RI4,RI4,1.0,AMAGP)

      IF (XSTD.LT.0.0) THEN
C	Apply tight mask to image used for refinement.
      	CALL MASKENV(NSAM,RIHP,OUTD,B3DV,0.0,0.0,PHI,THETA,PSI,
     +               AMAGP,W,IBUF)
C	Also to apply tight mask to image which will be used for 3D 
C	reconstruction.... might not be such a good idea, who knows...
C      	CALL MASKENV(NSAM,RIHP,DATD,B3DV,0.0,0.0,PHI,THETA,PSI,
c     +               AMAGP,W,IBUF)
      ENDIF

C      CALL RLFT3(DATD,DATQ,NSAM,NSAM,1,1)
      CALL RLFT3(OUTD,OUTQ,NSAM,NSAM,1,1)
C      write(*,*)'after RLFT3s'

C     SHIFT PARTICLE TO ORIGIN
      DO 110 L=1,JC
        LL=L-1
        DO 110 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
      	  ISUM=(LL+MM)
      	  PSHFTR=1.0
      	  IF (MOD(ISUM,2).NE.0) PSHFTR=-1.0
          IF (L.NE.JC) THEN
      	    ID=L+NSAMH*(M-1)
            DATC(ID)=DATC(ID)*PSHFTR
            OUTC(ID)=OUTC(ID)*PSHFTR
          ELSE
            DATQ(M)=DATQ(M)*PSHFTR
            OUTQ(M)=OUTQ(M)*PSHFTR
          ENDIF
110   CONTINUE

      RETURN
      END
